"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = session;

var _utils = require("../lib/utils");

async function session(params) {
  const {
    options,
    sessionToken
  } = params;
  const {
    adapter,
    jwt,
    events,
    callbacks,
    logger
  } = options;
  const useJwtSession = options.session.jwt;
  const sessionMaxAge = options.session.maxAge;

  if (!sessionToken) {
    return {
      json: {}
    };
  }

  let sessionCookie;
  let json = {};

  if (useJwtSession) {
    try {
      var _events$session;

      const decodedToken = await jwt.decode({ ...jwt,
        token: sessionToken
      });
      const newExpires = (0, _utils.fromDate)(sessionMaxAge);
      const defaultSession = {
        user: {
          name: decodedToken === null || decodedToken === void 0 ? void 0 : decodedToken.name,
          email: decodedToken === null || decodedToken === void 0 ? void 0 : decodedToken.email,
          image: decodedToken === null || decodedToken === void 0 ? void 0 : decodedToken.picture
        },
        expires: newExpires.toISOString()
      };
      const token = await callbacks.jwt({
        token: decodedToken
      });
      const session = await callbacks.session({
        session: defaultSession,
        token
      });
      json = session;
      const newToken = await jwt.encode({ ...jwt,
        token
      });
      sessionCookie = {
        name: options.cookies.sessionToken.name,
        value: newToken,
        options: {
          expires: newExpires,
          ...options.cookies.sessionToken.options
        }
      };
      await ((_events$session = events.session) === null || _events$session === void 0 ? void 0 : _events$session.call(events, {
        session,
        token
      }));
    } catch (error) {
      logger.error("JWT_SESSION_ERROR", error);
      sessionCookie = {
        name: options.cookies.sessionToken.name,
        value: "",
        options: { ...options.cookies.sessionToken.options,
          maxAge: 0
        }
      };
    }
  } else {
    try {
      const {
        getSessionAndUser,
        deleteSession,
        updateSession
      } = adapter;
      let userAndSession = await getSessionAndUser(sessionToken);

      if (userAndSession && userAndSession.session.expires.valueOf() < Date.now()) {
        await deleteSession(sessionToken);
        userAndSession = null;
      }

      if (userAndSession) {
        var _events$session2;

        const {
          user,
          session
        } = userAndSession;
        const sessionUpdateAge = options.session.updateAge;
        const sessionIsDueToBeUpdatedDate = session.expires.valueOf() - sessionMaxAge * 1000 + sessionUpdateAge * 1000;
        const newExpires = (0, _utils.fromDate)(sessionMaxAge);

        if (sessionIsDueToBeUpdatedDate <= Date.now()) {
          await updateSession({
            sessionToken,
            expires: newExpires
          });
        }

        const sessionPayload = await callbacks.session({
          session: {
            user: {
              name: user.name,
              email: user.email,
              image: user.image
            },
            expires: session.expires.toISOString()
          },
          user
        });
        json = sessionPayload;
        sessionCookie = {
          name: options.cookies.sessionToken.name,
          value: sessionToken,
          options: {
            expires: newExpires,
            ...options.cookies.sessionToken.options
          }
        };
        await ((_events$session2 = events.session) === null || _events$session2 === void 0 ? void 0 : _events$session2.call(events, {
          session: sessionPayload
        }));
      } else if (sessionToken) {
        sessionCookie = {
          name: options.cookies.sessionToken.name,
          value: "",
          options: { ...options.cookies.sessionToken.options,
            maxAge: 0
          }
        };
      }
    } catch (error) {
      logger.error("SESSION_ERROR", error);
    }
  }

  return {
    json,
    cookie: sessionCookie
  };
}